/* Fsim is a combat simulator for the role-playing game FUDGE.  Some info
 * on FUDGE is found below.  This program is copyright (C) 1993 by Andy
 * Skinner.
 * Version: June 21, 1993
 */

/* FUDGE info: */

/**********************************************************************

	  FUDGE: Freeform, Universal, Donated Gaming Engine
		   A Free Role-playing Game (RPG).
			By Steffan O'Sullivan
                       Version: June 19, 1993

FUDGE is a freeform RPG "engine."  This means that the rules are
guidelines, not unbreakable laws.  Each GM *must* do a certain amount
of work customizing the rules to her tastes, supplying the body of the
gaming "vehicle" - FUDGE is just the engine.  This is unavoidable in a
freeform game that is trying to appeal to a wide variety of gamers.
Fortunately, this also means that supplements from *any* game company
are compatible with FUDGE!

FUDGE is specifically for people who want a good bedrock to build
their own system on.  If every commercial RPG details certain things
in ways you don't like, then FUDGE may be what you're looking for.  If
you have created a great game setting (or translated one from
fiction), but no other game system's rules seem to do it justice,
perhaps FUDGE can help you.

If, on the other hand, you demand fully fleshed-out rules or a world
background detailed by a game designer, then you may wish to read no
further . . .

In its current edition, FUDGE is intended for experienced Game
Masters.  That is, at this time there is no introductory "What is
role-playing" section, nor detailed explanations of how to handle any
given situation.  However, a glossary of FUDGE-specific terms is
presented at the beginning of each new major topic.  The game can work
with novice *players*, as long as the GM is experienced.

Note: the word "freeform" in this game has nothing to do with the live
role-playing style known in Australia as free-form.  In FUDGE, the
word simply means a flexible, simple, casual set of RPG rules,
requiring much to be filled in and improvised by the GM.

**********************************************************************

1.2  Legal Notice
-----------------

Version: June 19, 1993

FUDGE is copyright 1992, 1993 by Steffan O'Sullivan.  It may be freely
copied and distributed by any means desired.  This legal notice must
be included with each copy of FUDGE.  No charge may be made for FUDGE
beyond a maximum US$6 fee (at 1993 rates) for materials and shipping,
without written permission from the author.

A publisher who wishes to include FUDGE in a book of copyrighted
source material or adventures may do so, so long as (1) the
publisher's copyrighted material is longer than the amount of FUDGE
material included, (2) at least Chapter 1 of FUDGE, including this
legal notice and the full Table of Contents, is included, (3) the
words to Chapter 1 of FUDGE, including this legal notice, are not
changed in any way, and (4) there is no extra charge for including
FUDGE in the book.

FUDGE was first published in 1992 on internet, and was written by
Steffan O'Sullivan (sos@oz.plymouth.edu) with an extensive amount of
high-quality input from Andy Skinner.  Other valued contributors
include Martin Bergendahl, Thomas Brettinger, Robert Bridson, Travis
Casey, Paul Jason Clegg, Peter F. Delaney, Jay Doane, Shawn Garbett,
Ed Heil, Bernard Hsiung, John H. Kim, Christian Otkjaer, Bill Seurer,
Larry Smith, Stephan Szabo, and others on rec.games.design on
internet.

Groo the Wanderer (TM) is a trademark of Sergio Aragones, and use of
the name does not challenge the trademark status in any way.

**********************************************************************/

/* the program: */

#include <stdio.h>
#include <ctype.h>


/* defines for compiling on different systems
 * so far works on Sun, DEC (VAX?), and Amiga Lattice compiler
 * try compiling as is, then try switching between various options below
 */

/* there are two different string header files */
#include <strings.h>
/*
#include <string.h>
*/

/* case-ignoring string comparison functions */
#define CMPSTR(a,b) (strcasecmp((a), (b)) == 0)
/*
#define CMPSTR(a,b) (stricmp((a), (b)) == 0)
*/

/* random number functions--random() is better, but may not be available */
/* RANDOM(n) returns a random number between 1 and n, inclusive */
extern long random();
#define RANDOM(n) (random()%(n) + 1)
/*
extern long rand();
/* note: rand() has not-very random lower bits
#define RANDOM(n) ((rand()>>6)%(n) + 1)
*/

/* set random number seed--use whichever one goes with random function above */
extern int srandom();
#define SRANDOM(s) srandom(s)
/*
#define SRANDOM(s) srand(s)
*/

/* the random number seed--I would just use time(), but it can give two runs
   with the same "random" numbers if they are started within the same second
*/
#define SEED getpid()
/*
#include <time.h>
#define SEED time(NULL)
*/

/* my Amiga's compiler thinks main() should be a void function */
#define MAINRET int
/*#define MAINRET void */



/* This program may not explicitly use all these defines, but they give
   some idea of what the values mean. */

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define NCOMBATS 100		/* default number of combats */

#define NDAMAGE_LEVELS 7	/* including WELL and DEAD */
#define MAX_DAMAGE 18		/* maximum damage +1, actually */
#define MAX_ROUNDS 17		/* don't count more than this */

/* dice methods */
#define DICE_2D6 0
#define DICE_2D10 1
#define DICE_D2 2

/* action methods */
#define STANDARD 0		/* standard weapon vs weapon */
#define ALTERNATE 1		/* attack1 vs defense2, attack2 vs defense1 */

/* bonus methods */
#define BONUS_NULL 0		/* I needed an undefined value */
#define BONUS_ROLL 1		/* add bonus to the roll */
#define BONUS_LEVEL 2		/* add bonus to the level */

/* damage methods */
#define DAM_CONST 0		/* damage = subtraction of terms */
#define DAM_ROLL 1		/* roll for damage */
#define DAM_ROLL1 2		/* roll for damage */

#define ERROR (-999)
#define TERRIBLE (-3)
#define POOR (-2)
#define MEDIOCRE (-1)
#define FAIR (0)
#define GOOD (1)
#define GREAT (2)
#define SUPERB (3)
#define LEGENDARY (4)
#define NLEVELS 8

/* damage levels */
#define WELL 0
#define SCRATCH 1
#define HURT 2
#define VERY_HURT 3
#define INCAPACITATED 4
#define NEAR_DEATH 5
#define DEAD 6

/* structure holding data for a fighter and stats */
typedef struct
{
    char *name;
    int scale;			/* strength/size scale */
    int strength;		/* strength for your scale */
    int mass;			/* how much damage is done to you */
    int damage_capacity;	/* how damage levels are distributed */
    int weapon_skill;		/* */
    int weapon;			/* */
    int armor;			/* */
    int shield;			/* */
    int bonus;			/* added to all rolls--who knows why */
    int initiative;		/* */
    int knockout;		/* if non-zero, use knockout damage */

    int damage;			/* damage currently taken in points */
    int dlevel;			/* damage currently taken in levels */

    /* statistics */
    int wins, losses;
    int hits, misses;
    int win_cond[NDAMAGE_LEVELS];
    int lose_cond[NDAMAGE_LEVELS];
    int damage_count[MAX_DAMAGE];
} Fighter;

/* names of trait levels */
static char *level_string_array[NLEVELS] =
{
    "Terrible",
    "Poor",
    "Mediocre",
    "Fair",
    "Good",
    "Great",
    "Superb",
    "Legendary"
};

/* penalties for damage */
int penalty[NDAMAGE_LEVELS] = { 0, 0, -1, -2, -2, -2, -2 };

/* for a given damage_capacity, these are the damage levels */
int damage_table[NLEVELS][NDAMAGE_LEVELS] =
{
{ 1, 1, 1, 1, 1, 2, 0 },	/* Terrible */
{ 1, 2, 1, 1, 2, 2, 0 },	/* Poor */
{ 1, 2, 2, 1, 2, 2, 0 },	/* Mediocre */
{ 1, 2, 2, 2, 2, 2, 0 },	/* Fair */
{ 1, 3, 2, 2, 3, 2, 0 },	/* Good */
{ 1, 3, 3, 2, 3, 2, 0 },	/* Great */
{ 1, 3, 3, 3, 3, 2, 0 },	/* Superb */
{ 1, 4, 3, 3, 4, 2, 0 }		/* Legendary */
};

/* names of damage levels */
char *damage_string[NDAMAGE_LEVELS] =
{
    "Well", "Scratched", "Hurt", "Very Hurt", "Incapacitated", "Near Death",
    "Dead"
};

char *abbrev_damage[NDAMAGE_LEVELS] =
{
    "    Well", " Scratch", "    Hurt", " V. Hurt", " Incapac", "N. Death",
    "    Dead"
};

/* table for 2d10 dice method */
static int table10[] = { -5, -4, -4, -3, -3, -2, -2, -1, -1, 0, 0, 0, 0, 0,
			  1,  1,  2,  2,  3,  3,  4,  4,  5 };

/* global flags */
int print_chars = FALSE, verbose = FALSE;
int method = DICE_2D6;			/* dice method */
int action = STANDARD;			/* combat action method */
int bonus_method = BONUS_NULL;		/* when are bonuses added */
int end_damage = INCAPACITATED;		/* when to stop fight */
int damage_mode = DAM_CONST;		/* how to figure damage */

int first_hit_wins = 0;
int first_hurt_wins = 0;

/********************************************************************
 * names of skill and attribute levels
 */
char *level_string(i)
     int i;
{
    if (i == ERROR) return "ERROR";

    i -= TERRIBLE;

    if (i < 0 || i >= NLEVELS)
	return "ERROR";

    /* see level_string_array in static definitions above */
    return level_string_array[i];
}

/********************************************************************
 * set one fighter structure from another
 */
void set_character(fto, ffrom)
     Fighter *fto, *ffrom;
{
    bcopy((char *) ffrom, (char *) fto, sizeof(Fighter));
}


/********************************************************************
 * initialize a fighter structure with default values:
 *     Fair for attributes, Poor for skills
 */
void init_character(f)
     Fighter *f;
{
    int i;
    f->name = NULL;
    f->scale = 0;
    f->strength = FAIR;
    f->mass = FAIR;
    f->damage_capacity = FAIR;
    f->weapon_skill = POOR;
    f->weapon = 0;
    f->armor = 0;
    f->shield = 0;
    f->bonus = 0;
    f->initiative = FAIR;
    f->knockout = 0;

    f->damage = 0;
    f->dlevel = WELL;
    f->wins = f->losses = 0;
    f->hits = f->misses = 0;
    for (i = 0; i < NDAMAGE_LEVELS; i++)
	f->win_cond[i] = f->lose_cond[i] = 0;

    for (i = 0; i < MAX_DAMAGE; i++)
	f->damage_count[i] = 0;
}


/********************************************************************
 * given a string containing a level (numeric or string label), return
 * the number representing that level
 */
int read_level(str)
     char *str;
{
    char level[12];
    int tmp;

    sscanf(str, "%s", level);
    if (CMPSTR(level, "terrible"))
	return TERRIBLE;
    else if (CMPSTR(level, "poor"))
	return POOR;
    else if (CMPSTR(level, "mediocre"))
	return MEDIOCRE;
    else if (CMPSTR(level, "fair"))
	return FAIR;
    else if (CMPSTR(level, "good"))
	return GOOD;
    else if (CMPSTR(level, "great"))
	return GREAT;
    else if (CMPSTR(level, "superb"))
	return SUPERB;
    else if (CMPSTR(level, "legendary"))
	return LEGENDARY;

    /* maybe numeric value was entered */
    if (sscanf(str, "%d", &tmp) != 1) return ERROR;
    if (tmp < TERRIBLE) tmp = TERRIBLE;
    if (tmp > LEGENDARY) tmp = LEGENDARY;
    return tmp;
}


/**************************************************************************
 * skip a word in a string, returning a pointer to the next word
 */
char *skip_word(str, n)
     char *str;
     int n;
{
    if (str == NULL) return NULL;
    if (*str == '\0') return NULL;
    while (isspace(*str)) str++;
    while (n--)
    {
	if (*str == '\0') return NULL;
	while (!isspace(*str)) str++;
	if (*str == '\0') return NULL;
	while (isspace(*str)) str++;
    }
    if (*str == '\0') return NULL;
    return str;
}


/********************************************************************
 * given a file name, read in a character description
 * non-mentioned traits are unchanged, so call init_character() first
 * extra stuff is skipped, so can be used as comments
 */
void read_character(fname, f)
     char *fname;
     Fighter *f;
{
    FILE *fp;
    static char tmp[81], heading[40];
    char *c;
    int i;

    fp = fopen(fname, "r");
    if (fp == NULL)
    {
	fprintf(stderr, "Couldn't read character file '%s'\n", fname);
	return;
    }

    while (fgets(tmp, 80, fp))
    {
	if (sscanf(tmp, "%s", heading) != 1)
	    continue;

	if (CMPSTR(heading, "strength"))
	{
	    c = skip_word(tmp, 1);
	    if ((f->strength = read_level(c)) == ERROR)
	    {
		fprintf(stderr, "bad strength level: '%s'\n", c);
		f->strength = FAIR;
	    }
	}
	else if (CMPSTR(heading, "mass"))
	{
	    c = skip_word(tmp, 1);
	    if ((f->mass = read_level(c)) == ERROR)
	    {
		fprintf(stderr, "bad mass level: '%s'\n", c);
		f->mass = FAIR;
	    }
	}
	else if (CMPSTR(heading, "damage_capacity"))
	{
	    c = skip_word(tmp, 1);
	    if ((f->damage_capacity = read_level(c)) == ERROR)
	    {
		fprintf(stderr, "bad damage_capacity level: '%s'\n", c);
		f->damage_capacity = FAIR;
	    }
	}
	else if (CMPSTR(heading, "skill"))
	{
	    c = skip_word(tmp, 1);
	    if ((f->weapon_skill = read_level(c)) == ERROR)
	    {
		fprintf(stderr, "bad weapon_skill level: '%s'\n", c);
		f->weapon_skill = FAIR;
	    }
	}
	else if (CMPSTR(heading, "weapon"))
	{
	    c = skip_word(tmp, 1);
	    f->weapon = atoi(c);
	}
	else if (CMPSTR(heading, "armor"))
	{
	    /* actually just numeric value for now */
	    c = skip_word(tmp, 1);
	    f->armor = atoi(c);
	}
	else if (CMPSTR(heading, "shield"))
	{
	    c = skip_word(tmp, 1);
	    f->shield = atoi(c);
	}
	else if (CMPSTR(heading, "bonus"))
	{
	    c = skip_word(tmp, 1);
	    f->bonus = atoi(c);
	}
	else if (CMPSTR(heading, "initiative"))
	{
	    c = skip_word(tmp, 1);
	    if ((f->initiative = read_level(c)) == ERROR)
	    {
		fprintf(stderr, "bad initiative level: '%s'\n", c);
		f->initiative = FAIR;
	    }
	}
	else if (CMPSTR(heading, "scale"))
	{
	    c = skip_word(tmp, 1);
	    f->scale = atoi(c);
	}
	else if (CMPSTR(heading, "knockout"))
	{
	    f->knockout = 1;
	}
	else
	{
	    /* skip it */
	}
    }

    fclose(fp);
}


/********************************************************************
 * Print out a character
 */
void print_character(f)
     Fighter *f;
{
    printf("scale %+d\n", f->scale);
    printf("strength %s\n", level_string(f->strength));
    printf("mass %s\n", level_string(f->mass));
    printf("damage_capacity %s\n", level_string(f->damage_capacity));
    printf("skill %s\n", level_string(f->weapon_skill));
    printf("weapon %+d\n", f->weapon);
    printf("armor %+d\n", f->armor);
    printf("shield %+d\n", f->shield);
    printf("bonus %+d\n", f->bonus);
    printf("initiative %s\n", level_string(f->initiative));
}


/********************************************************************
 * compute the final result for 1 player (relative degree is difference
 * between rolled degrees) for a given skill level, any bonuses, the
 * dice method used: 2D6 or 2D10, and whether the bonus is applied to
 * the roll or the level.
 */
int rolled_degree(skill, bonus)
     int skill;			/* might add other skills */
     int bonus;
{
    int roll;

    if (method == DICE_2D6)
    {
	roll = RANDOM(6) + RANDOM(6);
	if (bonus_method == BONUS_ROLL)
	    roll += bonus;

	if (roll < 6) roll -= 6;
	else if (roll > 8) roll -= 8;
	else roll = 0;
    }
    else if (method == DICE_2D10)
    {
	roll = RANDOM(10) + RANDOM(10);

	if (bonus_method == BONUS_ROLL)
	    roll += bonus;

	if (roll < 0) roll = 0;
	else if (roll > 22) roll = 22;
	roll = table10[roll];
    }
    else if (method == DICE_D2)
    {
	int roll1, roll2, increment;
	roll = 0;
	roll1 = RANDOM(2);
	increment = roll1 ? 1 : -1;
	while (1)
	{
	    roll2 = RANDOM(2);
	    if (roll1 == roll2)
		roll += increment;
	    else break;
	}
    }

    if (bonus_method == BONUS_LEVEL)
	roll += bonus;
    return skill + roll;
}



/********************************************************************/
int calc_damage_level(damage_capacity, damage)
     int damage_capacity;		/* */
     int damage;		/* points taken */
{
    int i = 0;

    if (damage_capacity < TERRIBLE || damage_capacity > LEGENDARY)
	return ERROR;

    while (damage > 0 && i < NDAMAGE_LEVELS-1)
    {
	damage -= damage_table[damage_capacity-TERRIBLE][i];
	if (damage >= 0) i++;
    }
    return i;
}

/********************************************************************/
int assess_damage(f_a, f_d, rel_degree)
     Fighter *f_a;		/* attacker */
     Fighter *f_d;		/* defender */
     int rel_degree;
{
    int damage, add, reduce;

    add = rel_degree + f_a->weapon + f_a->scale + f_a->strength;

    reduce = f_d->scale + f_d->mass - f_d->armor - f_d->shield;

    damage = add - reduce;

    if (damage_mode == DAM_CONST)
    {
	if (verbose)
	    printf(" for %d-%d=%d", add, reduce, damage);
    }
    else if (damage_mode == DAM_ROLL)
    {
	int roll_d = rolled_degree(0, 0);
	if (damage <= 0)	/* negative calculated damage can't give > 1 */
	{
	    damage += roll_d;
	    if (damage > 1) damage = 1;
	}
	else			/* positive calc damage can't *= 2 can't < 1 */
	{
	    if (roll_d > damage) roll_d = damage;
	    damage += roll_d;
	    if (damage < 1) damage = 1;
	}
	if (verbose)
	    printf(" for %d-%d%+d=%d", add, reduce, roll_d, damage);
    }
    else if (damage_mode == DAM_ROLL1)
    {
	/* damage roll without constraints */
	int roll_d = rolled_degree(0, 0);
	damage += roll_d;
	if (verbose)
	    printf(" for %d-%d%+d=%d", add, reduce, roll_d, damage);
    }

    if (damage > MAX_DAMAGE-1) damage = MAX_DAMAGE-1;
    else if (damage < 0) damage = 0;

    if (verbose)
	printf(" : %d points of damage", damage);

    f_d->damage += damage;
    f_d->dlevel = calc_damage_level(f_d->damage_capacity, f_d->damage);

    /* count how many times you do how much damage */
    f_a->damage_count[damage]++;

    if (verbose)
    {
	if (damage != 0)
	    printf(": %s\n", damage_string[f_d->dlevel]);
	else
	    printf("\n");
    }

    return damage;
}


/********************************************************************/
/* f1 always goes first--figure initiative before calling this function */
/* attack1 vs defense2, attack2 vs defense1 */
int acombat(f1, f2, rounds, hits)
     Fighter *f1, *f2;
     int *rounds, *hits;
{
    int degree1_a, degree1_d, degree2_a, degree2_d;
    int winner, first_blood = -1, first_hurt = -1;
    int defense;
    int d;

    *rounds = *hits = 0;

    /* while both are still able to fight, keep fighting */
    while (f1->dlevel < end_damage && f2->dlevel < end_damage)
    {
	(*rounds)++;

	degree1_a = rolled_degree(f1->weapon_skill,
				  f2->knockout ? f1->bonus :
				  penalty[f1->dlevel]+f1->bonus);
	defense = f1->knockout ? 0 : penalty[f2->dlevel];
	degree2_d = rolled_degree(f2->weapon_skill, defense+f2->bonus);

	/* rolls are compared to weapon_skills */
	if (degree1_a > degree2_d)
	{
	    /* fighter 1 did better than fighter 2 */
	    if (verbose)
		printf("%s hits %s", f1->name, f2->name);
	    d = assess_damage(f1, f2, degree1_a - degree2_d);
	    if (d > 0 && first_blood < 0) first_blood = 1;
	    if (d > 0 && first_hurt < 0 && penalty[f2->dlevel] < 0)
		first_hurt = 1;
	    f1->hits++;
	    (*hits)++;
	}
	else
	    f1->misses++;

	/* fight could be over here */
	if (f2->dlevel >= end_damage) break;

	degree2_a = rolled_degree(f2->weapon_skill,
				  f1->knockout ? f2->bonus :
				  penalty[f2->dlevel] + f2->bonus);
	defense = f2->knockout ? 0 : penalty[f1->dlevel];
	degree1_d = rolled_degree(f1->weapon_skill, defense + f1->bonus);
	if (degree2_a > degree1_d)
	{
	    /* fighter 2 did better than fighter 1 */
	    if (verbose)
		printf("%s hits %s", f2->name, f1->name);
	    d = assess_damage(f2, f1, degree2_a - degree1_d);
	    if (d > 0 && first_blood < 0) first_blood = 2;
	    if (d > 0 && first_hurt < 0 && penalty[f1->dlevel] < 0)
		first_hurt = 2;
	    f2->hits++;
	    (*hits)++;
	}
	else
	    f2->misses++;
    }

    /* can't be a tie here */
    winner = 0;

    if (f1->dlevel < end_damage)
	winner = 1;
    else if (f2->dlevel < end_damage)
	winner = 2;

    if (winner == 1)
    {
	f1->win_cond[f1->dlevel]++;
	f1->wins++;
	f2->lose_cond[f2->dlevel]++;
	if (verbose)
	    printf("%s, who is %s, wins the fight, leaving %s %s.\n",
		   f1->name, damage_string[f1->dlevel],
		   f2->name, damage_string[f2->dlevel]);
    }
    else if (winner == 2)
    {
	f2->win_cond[f2->dlevel]++;
	f2->wins++;
	f1->lose_cond[f1->dlevel]++;
	if (verbose)
	    printf("%s, who is %s, wins the fight, leaving %s %s.\n",
		   f2->name, damage_string[f2->dlevel],
		   f1->name, damage_string[f1->dlevel]);
    }

    if (winner == first_blood)
	first_hit_wins++;
    if (winner == first_hurt)
	first_hurt_wins++;

    return winner;
}

/********************************************************************/
int combat(f1, f2, rounds, hits)
     Fighter *f1, *f2;
     int *rounds, *hits;
{
    int degree1, degree2;
    int winner, first_blood = -1, first_hurt = -1;
    int d;

    *rounds = *hits = 0;

    /* while both are still able to fight, keep fighting */
    while (f1->dlevel < end_damage && f2->dlevel < end_damage)
    {
	(*rounds)++;

	degree1 = rolled_degree(f1->weapon_skill,
				f2->knockout ? f1->bonus :
				penalty[f1->dlevel] + f1->bonus);
	degree2 = rolled_degree(f2->weapon_skill,
				f1->knockout ? f2->bonus :
				penalty[f2->dlevel] + f2->bonus);

	if (degree1 > degree2)
	{
	    /* fighter 1 did better than fighter 2 */
	    if (verbose)
		printf("%s hits %s", f1->name, f2->name);
	    d = assess_damage(f1, f2, degree1 - degree2);
	    if (d > 0 && first_blood < 0) first_blood = 1;
	    if (d > 0 && first_hurt < 0 && penalty[f2->dlevel] < 0)
		first_hurt = 1;
	    f1->hits++; f2->misses++;
	    (*hits)++;
	}
	else if (degree2 > degree1)
	{
	    /* fighter 2 did better than fighter 1 */
	    if (verbose)
		printf("%s hits %s", f2->name, f1->name);
	    d = assess_damage(f2, f1, degree2 - degree1);
	    if (d > 0 && first_blood < 0) first_blood = 2;
	    if (d > 0 && first_hurt < 0 && penalty[f1->dlevel] < 0)
		first_hurt = 2;
	    f2->hits++; f1->misses++;
	    (*hits)++;
	}
	else
	{
	    if (verbose)
		printf("%s and %s tie\n", f1->name, f2->name);
	    /* both missed */
	    f1->misses++;
	    f2->misses++;
	    continue;
	}
    }

    /* no tie */
    winner = 0;

    if (f1->dlevel >= end_damage)
	f1->lose_cond[f1->dlevel]++;
    else
    {
	f1->win_cond[f1->dlevel]++;
	f1->wins++;
	winner = 1;
	if (verbose)
	    printf("%s, who is %s, wins the fight, leaving %s %s.\n",
		   f1->name, damage_string[f1->dlevel],
		   f2->name, damage_string[f2->dlevel]);
    }

    if (f2->dlevel >= end_damage)
	f2->lose_cond[f2->dlevel]++;
    else
    {
	f2->win_cond[f2->dlevel]++;
	f2->wins++;
	winner = 2;
	if (verbose)
	    printf("%s, who is %s, wins the fight, leaving %s %s.\n",
		   f2->name, damage_string[f2->dlevel],
		   f1->name, damage_string[f1->dlevel]);
    }

    if (winner == first_blood)
	first_hit_wins++;
    if (winner == first_hurt)
	first_hurt_wins++;

    return winner;
}

/********************************************************************/
void print_percent(count, total, n)
     int count, total;
     int n;		/* number of places */
{
    int p;
    if (total == 0)
    {
	printf("%*.*s", n, n, "0");
	return;
    }

    p = (count*100 + total/2) / total;	/* an attempt at rounding */

    if (p == 0 && count > 0)
	printf("%*.*s", n, n, "+0");
    else
	printf("%*d", n, p);
}

/********************************************************************/
/* Fsim [options] [fighter 1] [fighter 2] */
MAINRET main(argc, argv)
     int argc;
     char *argv[];
{
    char *fname1 = NULL, *fname2 = NULL;	/* file names */
    Fighter F1, F2;				/* store the characters */
    Fighter f1, f2;				/* the ones used in combat */
    int i, j, k;
    int n = NCOMBATS;				/* number of times to run */
    int rounds_per_combat[MAX_ROUNDS];
    int hits_per_combat[MAX_ROUNDS];
    int rounds, hits;
    int total_rounds = 0;

    /* seed random number generator */
    SRANDOM(SEED);

    for (i = 0; i < MAX_ROUNDS; i++)
	hits_per_combat[i] = rounds_per_combat[i] = 0;

    /* parse command line arguments */
    i = 1; j = 0;
    while (i < argc)
    {
	if (strcmp(argv[i], "-n") == 0)
	{
	    n = atoi(argv[++i]);
	    if (n == 0)
	    {
		fprintf(stderr, "Bad number of combats: '%s'\n", argv[i]);
		n = NCOMBATS;
	    }
	}
	else if (strcmp(argv[i], "-2") == 0)
	    method = DICE_D2;
	else if (strcmp(argv[i], "-6") == 0)
	    method = DICE_2D6;
	else if (strcmp(argv[i], "-10") == 0)
	    method = DICE_2D10;
	else if (strcmp(argv[i], "-p") == 0)
	    print_chars = TRUE;
	else if (strcmp(argv[i], "-v") == 0)
	    verbose = TRUE;
	else if (strcmp(argv[i], "-a") == 0)
	    action = ALTERNATE;
	else if (strcmp(argv[i], "-r") == 0)
	    bonus_method = BONUS_ROLL;
	else if (strcmp(argv[i], "-l") == 0)
	    bonus_method = BONUS_LEVEL;
	else if (strcmp(argv[i], "-d") == 0)
	{
	    i++;
	    /* set combat-ending damage level */
	    for (k = 0; k < NDAMAGE_LEVELS; k++)
	    {
		if (CMPSTR(argv[i], damage_string[k]))
		    end_damage = k;
	    }
	}
	else if (strcmp(argv[i], "-R") == 0)
	    damage_mode = DAM_ROLL;
	else if (strcmp(argv[i], "-R1") == 0)
	    damage_mode = DAM_ROLL1;
	else if (strcmp(argv[i], "-h") == 0)
	{
	    printf("usage: Fsim [options] [fighter 1] [fighter 2]\n");
	    printf("   -h            : print this message\n");
	    printf("   -n <number>   : number of times to fight\n");
	    printf("   -2            : use head/tail dice/coin method\n");
	    printf("   -6            : use 2D6 dice method\n");
	    printf("   -10           : use 2D10 dice method\n");
	    printf("   -l            : add bonuses to the skill level\n");
	    printf("   -r            : add bonuses to the roll\n");
	    printf("   -p            : print out the character files first\n");
	    printf("   -v            : verbose, blow by blow account\n");
	    printf("   -a            : use alternate opposed combat method\n");
	    printf("   -d <dam. lev.>: stop fight at this damage level\n");
	    printf("   -R            : use damage roll (default is don't)\n");
	    printf("   -R1           : damage roll, no constraints\n");
	    exit(0);
	}
	else
	{
	    if (argv[i][0] == '-')
		fprintf(stderr, "***unknown option: '%s'\n", argv[i]);
	    else
	    {
		if (j == 0)
		    fname1 = argv[i];
		else if (j == 1)
		    fname2 = argv[i];
		else
		    fprintf(stderr, "***too many characters specified\n");
		j++;
	    }
	}
	i++;
    }

    if (bonus_method == BONUS_NULL)
    {
	if (method == DICE_2D6)
	    bonus_method = BONUS_LEVEL;
	else
	    bonus_method = BONUS_ROLL;
    }

    /* no choice for this method */
    if (method == DICE_D2) bonus_method = BONUS_LEVEL;

    init_character(&F1);
    if (fname1)
	read_character(fname1, &F1);
    else fname1 = "Default";

    init_character(&F2);
    if (fname2)
	read_character(fname2, &F2);
    else fname2 = "Default";

    set_character(&f1, &F1);
    set_character(&f2, &F2);
    f1.name = fname1; f2.name = fname2;

    /* fight, keep statistics on who won, what condition was winner, what
       condition was loser, how many exchanges, how many successful blows */

    if (print_chars)
    {
	printf("%s\n", fname1);
	print_character(&F1);
	printf("\n");
	printf("%s\n", fname2);
	print_character(&F2);
	printf("\n");
    }

    i = n;
    while (i--)
    {
	/* reset characters */
	f1.damage = F1.damage;
	f1.dlevel = F1.dlevel;
	f2.damage = F2.damage;
	f2.dlevel = F2.dlevel;

	if (action == STANDARD)
	    (void) combat(&f1, &f2, &rounds, &hits);
	else if (action == ALTERNATE)
	{
	    int init1, init2;
	    /* roll initiative */
	    do
	    {
		init1 = rolled_degree(f1.initiative, 0);
		init2 = rolled_degree(f2.initiative, 0);
	    } while (init1 == init2);
	    if (init1 < init2)
		(void) acombat(&f2, &f1, &rounds, &hits);
	    else
		(void) acombat(&f1, &f2, &rounds, &hits);
	}

	total_rounds += rounds;

	if (rounds > MAX_ROUNDS) rounds = MAX_ROUNDS;
	rounds_per_combat[rounds-1]++;
	if (hits > MAX_ROUNDS) hits = MAX_ROUNDS;
	hits_per_combat[hits-1]++;
	if (verbose)
	    printf("\n");
    }


    /* print win scores */
    printf("A: %10s won ", fname1);
    print_percent(f1.wins, n, 3);
    printf("%%: %d\n", f1.wins);
    printf("B: %10s won ", fname2);
    print_percent(f2.wins, n, 3);
    printf("%%: %d\n", f2.wins);

    printf("\n");

    /* print how many times each fighter finished in each condition */
    printf("Final");
    for (i = WELL; i <= DEAD; i++)
    {
	printf("%8.8s ", abbrev_damage[i]);
	if (i != DEAD) printf("|");
	if (i == end_damage - 1) printf("|");
    }
    printf("\n");

    printf("A:   ");
    for (i = WELL; i <= DEAD; i++)
    {
	if (i < end_damage)
	    print_percent(f1.win_cond[i], n, 8);
	else
	    print_percent(f1.lose_cond[i], n, 8);
	if (i != DEAD) printf(" |");
	if (i == end_damage - 1) printf("|");
    }
    printf("\n");

    printf("B:   ");
    for (i = WELL; i <= DEAD; i++)
    {
	if (i < end_damage)
	    print_percent(f2.win_cond[i], n, 8);
	else
	    print_percent(f2.lose_cond[i], n, 8);
	if (i != DEAD) printf(" |");
	if (i == end_damage - 1) printf("|");
    }
    printf("\n");

    printf("\n");
    printf("The fighter who drew blood first won ");
    print_percent(first_hit_wins, n, 3);
    printf("%% of the time\n");

    printf("The fighter who first Hurt the other won ");
    print_percent(first_hurt_wins, n, 3);
    printf("%% of the time\n\n");

    /* print amounts of damage and how many times each fighter scored that */

    printf("Misses, Hits, Attempts   A: %d, %d, %d    B: %d, %d, %d\n",
	   f1.misses, f1.hits, f1.misses+f1.hits,
	   f2.misses, f2.hits, f2.misses+f2.hits);

    printf("Damage:\n");

    printf("  ");
    for (i = 0; i < MAX_DAMAGE; i++)
	printf("%4d", i);
    printf("+\n");

    printf("A:");
    for (i = 0; i < MAX_DAMAGE; i++)
	print_percent(f1.damage_count[i], f1.hits, 4);
    printf("\n");

    printf("B:");
    for (i = 0; i < MAX_DAMAGE; i++)
	print_percent(f2.damage_count[i], f2.hits, 4);
    printf("\n");

    printf("\n");

    i = f1.misses + f1.hits;
    j = f2.misses + f2.hits;
    if (i < total_rounds)
    {
	printf("%s lost an attack in ", fname1);
	print_percent(total_rounds - i, n, 3);
	printf("%% of combats\n");
    }
    if (j < total_rounds)
    {
	printf("%s lost an attack in ", fname2);
	print_percent(total_rounds - j, n, 3);
	printf("%% of combats\n");
    }

    /* print how many combats lasted how many rounds, and how many hits made */

    printf("        ");
    for (i = 0; i < MAX_ROUNDS; i++)
	printf("%4d", i+1);
    printf("+\n");

    printf("rounds: ");
    for (i = 0; i < MAX_ROUNDS; i++)
	print_percent(rounds_per_combat[i], n, 4);
    printf("\n");

    printf("hits  : ");
    for (i = 0; i < MAX_ROUNDS; i++)
	print_percent(hits_per_combat[i], n, 4);
    printf("\n");
}
